@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* session.c: save/restore session data */
@@ -220,7 +219,7 @@ session_save_servers (FILE *file)
         rc = rc && (session_write_int (file, SESSION_SERV_AUTOCONNECT, ptr_server->autoconnect));
         rc = rc && (session_write_int (file, SESSION_SERV_AUTORECONNECT, ptr_server->autoreconnect));
         rc = rc && (session_write_int (file, SESSION_SERV_AUTORECONNECT_DELAY, ptr_server->autoreconnect_delay));
-        rc = rc && (session_write_int (file, SESSION_SERV_COMMAND_LINE, ptr_server->command_line));
+        rc = rc && (session_write_int (file, SESSION_SERV_TEMP_SERVER, ptr_server->temp_server));
         rc = rc && (session_write_str (file, SESSION_SERV_ADDRESS, ptr_server->address));
         rc = rc && (session_write_int (file, SESSION_SERV_PORT, ptr_server->port));
         rc = rc && (session_write_int (file, SESSION_SERV_IPV6, ptr_server->ipv6));
@@ -261,6 +260,7 @@ session_save_servers (FILE *file)
         rc = rc && (session_write_str (file, SESSION_SERV_NICK_MODES, ptr_server->nick_modes));
         rc = rc && (session_write_str (file, SESSION_SERV_PREFIX, ptr_server->prefix));
         rc = rc && (session_write_buf (file, SESSION_SERV_RECONNECT_START, &(ptr_server->reconnect_start), sizeof (time_t)));
+        rc = rc && (session_write_buf (file, SESSION_SERV_COMMAND_TIME, &(ptr_server->command_time), sizeof (time_t)));
         rc = rc && (session_write_int (file, SESSION_SERV_RECONNECT_JOIN, ptr_server->reconnect_join));
         rc = rc && (session_write_int (file, SESSION_SERV_IS_AWAY, ptr_server->is_away));
         rc = rc && (session_write_str (file, SESSION_SERV_AWAY_MESSAGE, ptr_server->away_message));
@@ -295,7 +295,7 @@ session_save_dcc (FILE *file)
     
     rc = 1;
     
-    for (ptr_dcc = last_dcc; ptr_dcc;
+    for (ptr_dcc = irc_last_dcc; ptr_dcc;
          ptr_dcc = ptr_dcc->prev_dcc)
     {
         rc = rc && (session_write_id  (file, SESSION_OBJ_DCC));
@@ -399,8 +399,8 @@ session_save_buffers (FILE *file)
          ptr_buffer = ptr_buffer->next_buffer)
     {
         rc = rc && (session_write_id  (file, SESSION_OBJ_BUFFER));
-        rc = rc && (session_write_str (file, SESSION_BUFF_SERVER, SERVER(ptr_buffer) ? SERVER(ptr_buffer)->name : NULL));
-        rc = rc && (session_write_str (file, SESSION_BUFF_CHANNEL, CHANNEL(ptr_buffer) ? CHANNEL(ptr_buffer)->name : NULL));
+        rc = rc && (session_write_str (file, SESSION_BUFF_SERVER, GUI_SERVER(ptr_buffer) ? GUI_SERVER(ptr_buffer)->name : NULL));
+        rc = rc && (session_write_str (file, SESSION_BUFF_CHANNEL, GUI_CHANNEL(ptr_buffer) ? GUI_CHANNEL(ptr_buffer)->name : NULL));
         rc = rc && (session_write_int (file, SESSION_BUFF_TYPE, ptr_buffer->type));
         rc = rc && (session_write_int (file, SESSION_BUFF_ALL_SERVERS, ptr_buffer->all_servers));
         rc = rc && (session_write_id  (file, SESSION_BUFF_END));
@@ -450,13 +450,14 @@ session_save_hotlist (FILE *file)
     
     rc = 1;
     
-    for (ptr_hotlist = hotlist; ptr_hotlist;
+    for (ptr_hotlist = weechat_hotlist; ptr_hotlist;
          ptr_hotlist = ptr_hotlist->next_hotlist)
     {
         rc = rc && (session_write_id  (file, SESSION_OBJ_HOTLIST));
         rc = rc && (session_write_int (file, SESSION_HOTL_PRIORITY, ptr_hotlist->priority));
         rc = rc && (session_write_str (file, SESSION_HOTL_SERVER, (ptr_hotlist->server) ? ptr_hotlist->server->name : NULL));
         rc = rc && (session_write_int (file, SESSION_HOTL_BUFFER_NUMBER, ptr_hotlist->buffer->number));
+        rc = rc && (session_write_buf (file, SESSION_HOTL_CREATION_TIME, &(ptr_hotlist->creation_time), sizeof (struct timeval)));
         rc = rc && (session_write_id  (file, SESSION_HOTL_END));
         
         if (!rc)
@@ -881,8 +882,8 @@ session_load_server (FILE *file)
             case SESSION_SERV_AUTORECONNECT_DELAY:
                 rc = rc && (session_read_int (file, &(session_current_server->autoreconnect_delay)));
                 break;
-            case SESSION_SERV_COMMAND_LINE:
-                rc = rc && (session_read_int (file, &(session_current_server->command_line)));
+            case SESSION_SERV_TEMP_SERVER:
+                rc = rc && (session_read_int (file, &(session_current_server->temp_server)));
                 break;
             case SESSION_SERV_ADDRESS:
                 rc = rc && (session_read_str (file, &(session_current_server->address)));
@@ -993,6 +994,9 @@ session_load_server (FILE *file)
             case SESSION_SERV_RECONNECT_START:
                 rc = rc && (session_read_buf (file, &(session_current_server->reconnect_start), sizeof (time_t)));
                 break;
+            case SESSION_SERV_COMMAND_TIME:
+                rc = rc && (session_read_buf (file, &(session_current_server->command_time), sizeof (time_t)));
+                break;
             case SESSION_SERV_RECONNECT_JOIN:
                 rc = rc && (session_read_int (file, &(session_current_server->reconnect_join)));
                 break;
@@ -1165,7 +1169,7 @@ session_load_nick (FILE *file)
     
     /* allocate nick */
     nick = irc_nick_new (session_current_server, session_current_channel,
-                         nick_name, 0, 0, 0, 0, 0, 0);
+                         nick_name, 0, 0, 0, 0, 0, 0, 0);
     free (nick_name);
     if (!nick)
     {
@@ -1654,12 +1658,15 @@ session_load_hotlist (FILE *file)
 {
     int object_id, rc;
     int priority;
+    struct timeval creation_time;
     char *server_name;
     t_irc_server *ptr_server;
     int buffer_number;
     t_gui_buffer *ptr_buffer;
 
     priority = 0;
+    creation_time.tv_sec = 0;
+    creation_time.tv_usec = 0;
     ptr_server = NULL;
     ptr_buffer = NULL;
     
@@ -1677,7 +1684,7 @@ session_load_hotlist (FILE *file)
         switch (object_id)
         {
             case SESSION_HOTL_END:
-                hotlist_add (priority, ptr_server, ptr_buffer, 1);
+                hotlist_add (priority, &creation_time, ptr_server, ptr_buffer, 1);
                 return 1;
             case SESSION_HOTL_PRIORITY:
                 rc = rc && (session_read_int (file, &priority));
@@ -1693,6 +1700,9 @@ session_load_hotlist (FILE *file)
                 rc = rc && (session_read_int (file, &buffer_number));
                 ptr_buffer = gui_buffer_search_by_number (buffer_number);
                 break;
+            case SESSION_HOTL_CREATION_TIME:
+                rc = rc && (session_read_buf (file, &creation_time, sizeof (struct timeval)));
+                break;
             default:
                 weechat_log_printf (_("session: warning: ignoring value from "
                                       "history (object id: %d)\n"),
@@ -1843,13 +1853,13 @@ session_load (char *filename)
     
     if (unlink (filename) < 0)
     {
-        irc_display_prefix (NULL, gui_current_window->buffer, PREFIX_ERROR);
+        irc_display_prefix (NULL, gui_current_window->buffer, GUI_PREFIX_ERROR);
         gui_printf_nolog (gui_current_window->buffer,
                           _("%s can't delete session file (%s)\n"),
                           WEECHAT_ERROR, filename);
     }
     
-    irc_display_prefix (NULL, gui_current_window->buffer, PREFIX_INFO);
+    irc_display_prefix (NULL, gui_current_window->buffer, GUI_PREFIX_INFO);
     gui_printf_nolog (gui_current_window->buffer,
                       _("Upgrade completed successfully\n"));
     
