@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* gui-curses-input: input display functions for Curses GUI */
@@ -49,7 +48,7 @@ gui_input_set_color (t_gui_window *window, int irc_color)
     int fg, bg;
     
     fg = gui_irc_colors[irc_color][0];
-    bg = gui_color[COLOR_WIN_INPUT]->background;
+    bg = gui_color[GUI_COLOR_WIN_INPUT]->background;
     
     irc_color %= GUI_NUM_IRC_COLORS;
     if (gui_irc_colors[irc_color][1] & A_BOLD)
@@ -78,7 +77,7 @@ gui_input_get_prompt_length (t_gui_window *window, char *nick)
     char *pos, saved_char;
     int char_size, length;
     
-    if (window->buffer->text_search != TEXT_SEARCH_DISABLED)
+    if (window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED)
     {
         if (window->buffer->text_search_exact)
             return utf8_width_screen (_("Text search (exact): "));
@@ -97,21 +96,21 @@ gui_input_get_prompt_length (t_gui_window *window, char *nick)
                 switch (pos[0])
                 {
                     case 'c': /* channel or server name */
-                        if (CHANNEL(window->buffer))
-                            length += utf8_width_screen (CHANNEL(window->buffer)->name);
+                        if (GUI_CHANNEL(window->buffer))
+                            length += utf8_width_screen (GUI_CHANNEL(window->buffer)->name);
                         else
                         {
-                            if (SERVER(window->buffer))
-                                length += utf8_width_screen (SERVER(window->buffer)->name);
+                            if (GUI_SERVER(window->buffer))
+                                length += utf8_width_screen (GUI_SERVER(window->buffer)->name);
                         }
                         pos++;
                         break;
                     case 'm': /* nick modes */
-                        if (SERVER(window->buffer) && SERVER(window->buffer)->is_connected)
+                        if (GUI_SERVER(window->buffer) && GUI_SERVER(window->buffer)->is_connected)
                         {
-                            if (SERVER(window->buffer)->nick_modes
-                                && SERVER(window->buffer)->nick_modes[0])
-                                length += strlen (SERVER(window->buffer)->nick_modes);
+                            if (GUI_SERVER(window->buffer)->nick_modes
+                                && GUI_SERVER(window->buffer)->nick_modes[0])
+                                length += strlen (GUI_SERVER(window->buffer)->nick_modes);
                         }
                         pos++;
                         break;
@@ -161,10 +160,10 @@ gui_input_draw_prompt (t_gui_window *window, char *nick)
     
     wmove (GUI_CURSES(window)->win_input, 0, 0);
     
-    if (window->buffer->text_search != TEXT_SEARCH_DISABLED)
+    if (window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED)
     {
         gui_window_set_weechat_color (GUI_CURSES(window)->win_input,
-                                      COLOR_WIN_INPUT);
+                                      GUI_COLOR_WIN_INPUT);
         wprintw (GUI_CURSES(window)->win_input, "%s",
                  (window->buffer->text_search_exact) ?
                  _("Text search (exact): ") : _("Text search: "));
@@ -181,27 +180,27 @@ gui_input_draw_prompt (t_gui_window *window, char *nick)
                 switch (pos[0])
                 {
                     case 'c': /* channel or server name */
-                        if (CHANNEL(window->buffer))
+                        if (GUI_CHANNEL(window->buffer))
                         {
                             gui_window_set_weechat_color (GUI_CURSES(window)->win_input,
-                                                          COLOR_WIN_INPUT_CHANNEL);
+                                                          GUI_COLOR_WIN_INPUT_CHANNEL);
                             buf = weechat_iconv_from_internal (NULL,
-                                                               CHANNEL(window->buffer)->name);
+                                                               GUI_CHANNEL(window->buffer)->name);
                             wprintw (GUI_CURSES(window)->win_input, "%s",
-                                     (buf) ? buf : CHANNEL(window->buffer)->name);
+                                     (buf) ? buf : GUI_CHANNEL(window->buffer)->name);
                             if (buf)
                                 free (buf);
                         }
                         else
                         {
-                            if (SERVER(window->buffer))
+                            if (GUI_SERVER(window->buffer))
                             {
                                 gui_window_set_weechat_color (GUI_CURSES(window)->win_input,
-                                                              COLOR_WIN_INPUT_SERVER);
+                                                              GUI_COLOR_WIN_INPUT_SERVER);
                                 buf = weechat_iconv_from_internal (NULL,
-                                                                   SERVER(window->buffer)->name);
+                                                                   GUI_SERVER(window->buffer)->name);
                                 wprintw (GUI_CURSES(window)->win_input, "%s",
-                                         (buf) ? buf : SERVER(window->buffer)->name);
+                                         (buf) ? buf : GUI_SERVER(window->buffer)->name);
                                 if (buf)
                                     free (buf);
                             }
@@ -209,22 +208,22 @@ gui_input_draw_prompt (t_gui_window *window, char *nick)
                         pos++;
                         break;
                     case 'm': /* nick modes */
-                        if (SERVER(window->buffer) && SERVER(window->buffer)->is_connected)
+                        if (GUI_SERVER(window->buffer) && GUI_SERVER(window->buffer)->is_connected)
                         {
-                            if (SERVER(window->buffer)->nick_modes
-                                && SERVER(window->buffer)->nick_modes[0])
+                            if (GUI_SERVER(window->buffer)->nick_modes
+                                && GUI_SERVER(window->buffer)->nick_modes[0])
                             {
                                 gui_window_set_weechat_color (GUI_CURSES(window)->win_input,
-                                                              COLOR_WIN_INPUT);
+                                                              GUI_COLOR_WIN_INPUT);
                                 wprintw (GUI_CURSES(window)->win_input, "%s",
-                                         SERVER(window->buffer)->nick_modes);
+                                         GUI_SERVER(window->buffer)->nick_modes);
                             }
                         }
                         pos++;
                         break;
                     case 'n': /* nick */
                         gui_window_set_weechat_color (GUI_CURSES(window)->win_input,
-                                                      COLOR_WIN_INPUT_NICK);
+                                                      GUI_COLOR_WIN_INPUT_NICK);
                         buf = weechat_iconv_from_internal (NULL, nick);
                         wprintw (GUI_CURSES(window)->win_input, "%s", (buf) ? buf : nick);
                         if (buf)
@@ -238,7 +237,7 @@ gui_input_draw_prompt (t_gui_window *window, char *nick)
                             saved_char = pos[char_size];
                             pos[char_size] = '\0';
                             gui_window_set_weechat_color (GUI_CURSES(window)->win_input,
-                                                          COLOR_WIN_INPUT_DELIMITERS);
+                                                          GUI_COLOR_WIN_INPUT_DELIMITERS);
                             wprintw (GUI_CURSES(window)->win_input, "%%%s", pos);
                             pos[char_size] = saved_char;
                             pos += char_size;
@@ -256,7 +255,7 @@ gui_input_draw_prompt (t_gui_window *window, char *nick)
                 saved_char = pos[char_size];
                 pos[char_size] = '\0';
                 gui_window_set_weechat_color (GUI_CURSES(window)->win_input,
-                                              COLOR_WIN_INPUT_DELIMITERS);
+                                              GUI_COLOR_WIN_INPUT_DELIMITERS);
                 wprintw (GUI_CURSES(window)->win_input, "%s", pos);
                 pos[char_size] = saved_char;
                 pos += char_size;
@@ -274,7 +273,7 @@ gui_input_draw_prompt (t_gui_window *window, char *nick)
 int
 gui_input_draw_text (t_gui_window *window, int input_width)
 {
-    char *ptr_start, *ptr_next, saved_char, *output;
+    char *ptr_start, *ptr_next, saved_char, *output, *ptr_string;
     int pos_mask, size, last_color, color, count_cursor, offset_cursor;
     
     ptr_start = utf8_add_offset (window->buffer->input_buffer,
@@ -284,14 +283,14 @@ gui_input_draw_text (t_gui_window *window, int input_width)
     count_cursor = window->buffer->input_buffer_pos -
         window->buffer->input_buffer_1st_display;
     offset_cursor = 0;
-    if (window->buffer->text_search != TEXT_SEARCH_DISABLED)
+    if (window->buffer->text_search != GUI_TEXT_SEARCH_DISABLED)
     {
         if (window->buffer->text_search_found)
             gui_window_set_weechat_color (GUI_CURSES(window)->win_input,
-                                          COLOR_WIN_INPUT);
+                                          GUI_COLOR_WIN_INPUT);
         else
             gui_window_set_weechat_color (GUI_CURSES(window)->win_input,
-                                          COLOR_WIN_INPUT_TEXT_NOT_FOUND);
+                                          GUI_COLOR_WIN_INPUT_TEXT_NOT_FOUND);
     }
     while ((input_width > 0) && ptr_start && ptr_start[0])
     {
@@ -301,7 +300,7 @@ gui_input_draw_text (t_gui_window *window, int input_width)
             saved_char = ptr_next[0];
             ptr_next[0] = '\0';
             size = ptr_next - ptr_start;
-            if (window->buffer->text_search == TEXT_SEARCH_DISABLED)
+            if (window->buffer->text_search == GUI_TEXT_SEARCH_DISABLED)
             {
                 if (window->buffer->input_buffer_color_mask[pos_mask] != ' ')
                     color = window->buffer->input_buffer_color_mask[pos_mask] - '0';
@@ -310,21 +309,41 @@ gui_input_draw_text (t_gui_window *window, int input_width)
                 if (color != last_color)
                 {
                     if (color == -1)
-                        gui_window_set_weechat_color (GUI_CURSES(window)->win_input, COLOR_WIN_INPUT);
+                        gui_window_set_weechat_color (GUI_CURSES(window)->win_input, GUI_COLOR_WIN_INPUT);
                     else
                         gui_input_set_color (window, color);
                 }
                 last_color = color;
             }
             output = weechat_iconv_from_internal (NULL, ptr_start);
-            wprintw (GUI_CURSES(window)->win_input, "%s", (output) ? output : ptr_start);
+            
+            ptr_string = (output) ? output : ptr_start;
+            
+            if ((((unsigned char)ptr_string[0]) < 32) && (!ptr_string[1]))
+            {
+                wattron (GUI_CURSES(window)->win_input, A_REVERSE);
+                wprintw (GUI_CURSES(window)->win_input, "%c",
+                         'A' + ((unsigned char)ptr_string[0]) - 1);
+                wattroff (GUI_CURSES(window)->win_input, A_REVERSE);
+                if (count_cursor > 0)
+                {
+                    offset_cursor++;
+                    count_cursor--;
+                }
+            }
+            else
+            {
+                wprintw (GUI_CURSES(window)->win_input, "%s", ptr_string);
+                if (count_cursor > 0)
+                {
+                    offset_cursor += utf8_width_screen (ptr_start);
+                    count_cursor--;
+                }
+            }
+            
             if (output)
                 free (output);
-            if (count_cursor > 0)
-            {
-                offset_cursor += utf8_width_screen (ptr_start);
-                count_cursor--;
-            }
+            
             ptr_next[0] = saved_char;
             ptr_start = ptr_next;
             pos_mask += size;
@@ -357,120 +376,140 @@ gui_input_draw (t_gui_buffer *buffer, int erase)
         if (ptr_win->buffer == buffer)
         {
             if (erase)
-                gui_window_curses_clear (GUI_CURSES(ptr_win)->win_input, COLOR_WIN_INPUT);
-            
-            switch (buffer->type)
+                gui_window_curses_clear (GUI_CURSES(ptr_win)->win_input, GUI_COLOR_WIN_INPUT);
+
+            if (gui_keyboard_paste_pending)
             {
-                case BUFFER_TYPE_STANDARD:
-                    if (buffer->has_input)
-                    {
-                        if (buffer->input_buffer_length == 0)
-                            buffer->input_buffer[0] = '\0';
-                        
-                        if (SERVER(buffer) && SERVER(buffer)->is_connected)
-                            ptr_nickname = (SERVER(buffer)->nick) ?
-                                SERVER(buffer)->nick : cfg_look_no_nickname;
-                        else
-                            ptr_nickname = cfg_look_no_nickname;
-                        
-                        prompt_length = gui_input_get_prompt_length (ptr_win, ptr_nickname);
-                        
-                        if (ptr_win->win_input_width - prompt_length < 3)
+                wmove (GUI_CURSES(ptr_win)->win_input, 0, 0);
+                gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_input,
+                                              GUI_COLOR_WIN_INPUT_ACTIONS);
+                gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
+                                    _("  Paste %d lines ? [ctrl-Y] Yes  [ctrl-N] No"),
+                                    gui_keyboard_get_paste_lines ());
+                wclrtoeol (GUI_CURSES(ptr_win)->win_input);
+                ptr_win->win_input_cursor_x = 0;
+                if (ptr_win == gui_current_window)
+                    move (ptr_win->win_input_y, ptr_win->win_input_x);
+            }
+            else
+            {
+                switch (buffer->type)
+                {
+                    case GUI_BUFFER_TYPE_STANDARD:
+                        if (buffer->has_input)
                         {
-                            prompt_length = 0;
-                            display_prompt = 0;
-                        }
-                        else
-                            display_prompt = 1;
+                            if (buffer->input_buffer_length == 0)
+                                buffer->input_buffer[0] = '\0';
                         
-                        if (buffer->input_buffer_pos - buffer->input_buffer_1st_display + 1 >
-                            ptr_win->win_input_width - prompt_length)
-                            buffer->input_buffer_1st_display = buffer->input_buffer_pos -
-                                (ptr_win->win_input_width - prompt_length) + 1;
-                        else
-                        {
-                            if (buffer->input_buffer_pos < buffer->input_buffer_1st_display)
-                                buffer->input_buffer_1st_display = buffer->input_buffer_pos;
+                            if (GUI_SERVER(buffer) && GUI_SERVER(buffer)->is_connected)
+                                ptr_nickname = (GUI_SERVER(buffer)->nick) ?
+                                    GUI_SERVER(buffer)->nick : cfg_look_no_nickname;
+                            else
+                                ptr_nickname = cfg_look_no_nickname;
+                        
+                            prompt_length = gui_input_get_prompt_length (ptr_win, ptr_nickname);
+                        
+                            if (ptr_win->win_input_width - prompt_length < 3)
+                            {
+                                prompt_length = 0;
+                                display_prompt = 0;
+                            }
+                            else
+                                display_prompt = 1;
+                        
+                            if (buffer->input_buffer_pos - buffer->input_buffer_1st_display + 1 >
+                                ptr_win->win_input_width - prompt_length)
+                                buffer->input_buffer_1st_display = buffer->input_buffer_pos -
+                                    (ptr_win->win_input_width - prompt_length) + 1;
                             else
                             {
-                                if ((buffer->input_buffer_1st_display > 0) &&
-                                    (buffer->input_buffer_pos -
-                                     buffer->input_buffer_1st_display + 1)
-                                    < ptr_win->win_input_width - prompt_length)
+                                if (buffer->input_buffer_pos < buffer->input_buffer_1st_display)
+                                    buffer->input_buffer_1st_display = buffer->input_buffer_pos;
+                                else
                                 {
-                                    buffer->input_buffer_1st_display =
-                                        buffer->input_buffer_pos -
-                                        (ptr_win->win_input_width - prompt_length) + 1;
-                                    if (buffer->input_buffer_1st_display < 0)
-                                        buffer->input_buffer_1st_display = 0;
+                                    if ((buffer->input_buffer_1st_display > 0) &&
+                                        (buffer->input_buffer_pos -
+                                         buffer->input_buffer_1st_display + 1)
+                                        < ptr_win->win_input_width - prompt_length)
+                                    {
+                                        buffer->input_buffer_1st_display =
+                                            buffer->input_buffer_pos -
+                                            (ptr_win->win_input_width - prompt_length) + 1;
+                                        if (buffer->input_buffer_1st_display < 0)
+                                            buffer->input_buffer_1st_display = 0;
+                                    }
                                 }
                             }
-                        }
-                        if (display_prompt)
-                            gui_input_draw_prompt (ptr_win, ptr_nickname);
+                            if (display_prompt)
+                                gui_input_draw_prompt (ptr_win, ptr_nickname);
                         
-                        gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_input, COLOR_WIN_INPUT);
-                        snprintf (format, 32, "%%-%ds", ptr_win->win_input_width - prompt_length);
-                        offset_cursor = 0;
-                        if (ptr_win == gui_current_window)
-                            offset_cursor = gui_input_draw_text (ptr_win,
-                                                                 ptr_win->win_input_width - prompt_length);
-                        else
-                            wprintw (GUI_CURSES(ptr_win)->win_input, format, "");
-                        wclrtoeol (GUI_CURSES(ptr_win)->win_input);
-                        ptr_win->win_input_cursor_x = prompt_length + offset_cursor;
-                        if (ptr_win == gui_current_window)
-                            move (ptr_win->win_input_y,
-                                  ptr_win->win_input_x + ptr_win->win_input_cursor_x);
-                    }
-                    break;
-                case BUFFER_TYPE_DCC:
-                    dcc_selected = (ptr_win->dcc_selected) ? (t_irc_dcc *) ptr_win->dcc_selected : dcc_list;
-                    wmove (GUI_CURSES(ptr_win)->win_input, 0, 0);
-                    if (dcc_selected)
-                    {
-                        switch (dcc_selected->status)
-                        {
-                            case DCC_WAITING:
-                                if (DCC_IS_RECV(dcc_selected->type))
-                                    gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
-                                                        _("  [A] Accept"));
-                                gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
-                                                    _("  [C] Cancel"));
-                                break;
-                            case DCC_CONNECTING:
-                            case DCC_ACTIVE:
-                                gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
-                                                    _("  [C] Cancel"));
-                                break;
-                            case DCC_DONE:
-                            case DCC_FAILED:
-                            case DCC_ABORTED:
-                                gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
-                                                    _("  [R] Remove"));
-                                break;
+                            gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_input, GUI_COLOR_WIN_INPUT);
+                            snprintf (format, 32, "%%-%ds", ptr_win->win_input_width - prompt_length);
+                            offset_cursor = 0;
+                            if (ptr_win == gui_current_window)
+                                offset_cursor = gui_input_draw_text (ptr_win,
+                                                                     ptr_win->win_input_width - prompt_length);
+                            else
+                                wprintw (GUI_CURSES(ptr_win)->win_input, format, "");
+                            wclrtoeol (GUI_CURSES(ptr_win)->win_input);
+                            ptr_win->win_input_cursor_x = prompt_length + offset_cursor;
+                            if (ptr_win == gui_current_window)
+                                move (ptr_win->win_input_y,
+                                      ptr_win->win_input_x + ptr_win->win_input_cursor_x);
                         }
-                    }
-                    gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
-                                        _("  [P] Purge old DCC"));
-                    gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
-                                        _("  [Q] Close DCC view"));
-                    wclrtoeol (GUI_CURSES(ptr_win)->win_input);
-                    ptr_win->win_input_cursor_x = 0;
-                    if (ptr_win == gui_current_window)
-                        move (ptr_win->win_input_y, ptr_win->win_input_x);
-                    break;
-                case BUFFER_TYPE_RAW_DATA:
-                    wmove (GUI_CURSES(ptr_win)->win_input, 0, 0);
-                    gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
-                                        _("  [C] Clear buffer"));
-                    gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
-                                        _("  [Q] Close raw data view"));
-                    wclrtoeol (GUI_CURSES(ptr_win)->win_input);
-                    ptr_win->win_input_cursor_x = 0;
-                    if (ptr_win == gui_current_window)
-                        move (ptr_win->win_input_y, ptr_win->win_input_x);
-                    break;
+                        break;
+                    case GUI_BUFFER_TYPE_DCC:
+                        dcc_selected = (ptr_win->dcc_selected) ? (t_irc_dcc *) ptr_win->dcc_selected : irc_dcc_list;
+                        wmove (GUI_CURSES(ptr_win)->win_input, 0, 0);
+                        gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_input,
+                                                      GUI_COLOR_WIN_INPUT_ACTIONS);
+                        if (dcc_selected)
+                        {
+                            switch (dcc_selected->status)
+                            {
+                                case IRC_DCC_WAITING:
+                                    if (IRC_DCC_IS_RECV(dcc_selected->type))
+                                        gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
+                                                            _("  [A] Accept"));
+                                    gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
+                                                        _("  [C] Cancel"));
+                                    break;
+                                case IRC_DCC_CONNECTING:
+                                case IRC_DCC_ACTIVE:
+                                    gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
+                                                        _("  [C] Cancel"));
+                                    break;
+                                case IRC_DCC_DONE:
+                                case IRC_DCC_FAILED:
+                                case IRC_DCC_ABORTED:
+                                    gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
+                                                        _("  [R] Remove"));
+                                    break;
+                            }
+                        }
+                        gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
+                                            _("  [P] Purge old DCC"));
+                        gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
+                                            _("  [Q] Close DCC view"));
+                        wclrtoeol (GUI_CURSES(ptr_win)->win_input);
+                        ptr_win->win_input_cursor_x = 0;
+                        if (ptr_win == gui_current_window)
+                            move (ptr_win->win_input_y, ptr_win->win_input_x);
+                        break;
+                    case GUI_BUFFER_TYPE_RAW_DATA:
+                        wmove (GUI_CURSES(ptr_win)->win_input, 0, 0);
+                        gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_input,
+                                                      GUI_COLOR_WIN_INPUT_ACTIONS);
+                        gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
+                                            _("  [C] Clear buffer"));
+                        gui_window_wprintw (GUI_CURSES(ptr_win)->win_input,
+                                            _("  [Q] Close raw data view"));
+                        wclrtoeol (GUI_CURSES(ptr_win)->win_input);
+                        ptr_win->win_input_cursor_x = 0;
+                        if (ptr_win == gui_current_window)
+                            move (ptr_win->win_input_y, ptr_win->win_input_x);
+                        break;
+                }
             }
             wrefresh (GUI_CURSES(ptr_win)->win_input);
             refresh ();
