@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* gui-gtk-input: user input functions for Gtk GUI */
@@ -72,20 +71,20 @@ gui_input_get_prompt_length (t_gui_window *window, char *nick)
                 switch (pos[0])
                 {
                     case 'c':
-                        if (CHANNEL(window->buffer))
-                            length += utf8_strlen (CHANNEL(window->buffer)->name);
+                        if (GUI_CHANNEL(window->buffer))
+                            length += utf8_strlen (GUI_CHANNEL(window->buffer)->name);
                         else
                         {
-                            if (SERVER(window->buffer))
-                                length += utf8_strlen (SERVER(window->buffer)->name);
+                            if (GUI_SERVER(window->buffer))
+                                length += utf8_strlen (GUI_SERVER(window->buffer)->name);
                         }
                         pos++;
                         break;
                     case 'm':
-                        if (SERVER(window->buffer))
+                        if (GUI_SERVER(window->buffer))
                         {
                             mode_found = 0;
-                            for (modes = SERVER(window->buffer)->nick_modes;
+                            for (modes = GUI_SERVER(window->buffer)->nick_modes;
                                  modes && modes[0]; modes++)
                             {
                                 if (modes[0] != ' ')
