@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* gui-window.c: window functions, used by all GUI */
@@ -51,6 +50,7 @@
 
 t_gui_buffer *gui_buffers = NULL;           /* pointer to first buffer      */
 t_gui_buffer *last_gui_buffer = NULL;       /* pointer to last buffer       */
+t_gui_buffer *gui_previous_buffer = NULL;   /* pointer to previous buffer   */
 t_gui_buffer *gui_buffer_before_dcc = NULL; /* buffer before dcc switch     */
 t_gui_buffer *gui_buffer_raw_data = NULL;   /* buffer with raw IRC data     */
 t_gui_buffer *gui_buffer_before_raw_data = NULL; /* buffer before raw switch*/
@@ -87,7 +87,7 @@ gui_buffer_new (t_gui_window *window, void *server, void *channel, int type,
 {
     t_gui_buffer *new_buffer, *ptr_buffer;
 #ifdef PLUGINS
-    char buffer_str[16];
+    char buffer_str[16], *argv[1] = { NULL };
 #endif
     
 #ifdef DEBUG
@@ -95,8 +95,8 @@ gui_buffer_new (t_gui_window *window, void *server, void *channel, int type,
 #endif
     
     /* use first buffer if no server is assigned to this buffer */
-    if ((type == BUFFER_TYPE_STANDARD) && gui_buffers
-        && (!SERVER(gui_buffers)) && (!channel))
+    if ((type == GUI_BUFFER_TYPE_STANDARD) && gui_buffers
+        && (!GUI_SERVER(gui_buffers)) && (!channel))
     {
         if (server)
             ((t_irc_server *)(server))->buffer = gui_buffers;
@@ -113,7 +113,7 @@ gui_buffer_new (t_gui_window *window, void *server, void *channel, int type,
     }
 
     /* use "all servers" buffer if found */
-    if (cfg_look_one_server_buffer && (type == BUFFER_TYPE_STANDARD) &&
+    if (cfg_look_one_server_buffer && (type == GUI_BUFFER_TYPE_STANDARD) &&
         gui_buffers && server && !channel)
     {
         ptr_buffer = gui_buffer_servers_search ();
@@ -138,18 +138,18 @@ gui_buffer_new (t_gui_window *window, void *server, void *channel, int type,
         new_buffer->all_servers = 0;
         new_buffer->channel = channel;
         new_buffer->type = type;
-        if (new_buffer->type == BUFFER_TYPE_RAW_DATA)
+        if (new_buffer->type == GUI_BUFFER_TYPE_RAW_DATA)
             gui_buffer_raw_data = new_buffer;
         /* assign buffer to server and channel */
         if (server && !channel)
         {
-            SERVER(new_buffer)->buffer = new_buffer;
+            GUI_SERVER(new_buffer)->buffer = new_buffer;
             new_buffer->all_servers = (cfg_look_one_server_buffer) ? 1 : 0;
         }
         if (!gui_buffers && cfg_look_one_server_buffer)
             new_buffer->all_servers = 1;
         if (channel)
-            CHANNEL(new_buffer)->buffer = new_buffer;
+            GUI_CHANNEL(new_buffer)->buffer = new_buffer;
         
         if (!window->buffer)
         {
@@ -173,18 +173,18 @@ gui_buffer_new (t_gui_window *window, void *server, void *channel, int type,
         /* create/append to log file */
         new_buffer->log_filename = NULL;
         new_buffer->log_file = NULL;
-        if ((cfg_log_auto_server && BUFFER_IS_SERVER(new_buffer))
-            || (cfg_log_auto_channel && BUFFER_IS_CHANNEL(new_buffer))
-            || (cfg_log_auto_private && BUFFER_IS_PRIVATE(new_buffer)))
+        if ((cfg_log_auto_server && GUI_BUFFER_IS_SERVER(new_buffer))
+            || (cfg_log_auto_channel && GUI_BUFFER_IS_CHANNEL(new_buffer))
+            || (cfg_log_auto_private && GUI_BUFFER_IS_PRIVATE(new_buffer)))
             gui_log_start (new_buffer);
         
         /* init input buffer */
-        new_buffer->has_input = (new_buffer->type == BUFFER_TYPE_STANDARD) ? 1 : 0;
+        new_buffer->has_input = (new_buffer->type == GUI_BUFFER_TYPE_STANDARD) ? 1 : 0;
         if (new_buffer->has_input)
         {
-            new_buffer->input_buffer_alloc = INPUT_BUFFER_BLOCK_SIZE;
-            new_buffer->input_buffer = (char *) malloc (INPUT_BUFFER_BLOCK_SIZE);
-            new_buffer->input_buffer_color_mask = (char *) malloc (INPUT_BUFFER_BLOCK_SIZE);
+            new_buffer->input_buffer_alloc = GUI_INPUT_BUFFER_BLOCK_SIZE;
+            new_buffer->input_buffer = (char *) malloc (GUI_INPUT_BUFFER_BLOCK_SIZE);
+            new_buffer->input_buffer_color_mask = (char *) malloc (GUI_INPUT_BUFFER_BLOCK_SIZE);
             new_buffer->input_buffer[0] = '\0';
             new_buffer->input_buffer_color_mask[0] = '\0';
         }
@@ -208,7 +208,7 @@ gui_buffer_new (t_gui_window *window, void *server, void *channel, int type,
         new_buffer->num_history = 0;
         
         /* text search */
-        new_buffer->text_search = TEXT_SEARCH_DISABLED;
+        new_buffer->text_search = GUI_TEXT_SEARCH_DISABLED;
         new_buffer->text_search_exact = 0;
         new_buffer->text_search_found = 0;
         new_buffer->text_search_input = NULL;
@@ -225,7 +225,7 @@ gui_buffer_new (t_gui_window *window, void *server, void *channel, int type,
         /* move buffer next to server */
         if (server && cfg_look_open_near_server && (!cfg_look_one_server_buffer))
         {
-            ptr_buffer = SERVER(new_buffer)->buffer;
+            ptr_buffer = GUI_SERVER(new_buffer)->buffer;
             while (ptr_buffer && (ptr_buffer->server == server))
             {
                 ptr_buffer = ptr_buffer->next_buffer;
@@ -243,7 +243,8 @@ gui_buffer_new (t_gui_window *window, void *server, void *channel, int type,
 
 #ifdef PLUGINS
         snprintf (buffer_str, sizeof (buffer_str) - 1, "%d", new_buffer->number);
-        (void) plugin_event_handler_exec ("buffer_open", buffer_str);
+        argv[0] = buffer_str;
+        (void) plugin_event_handler_exec ("buffer_open", 1, argv);
 #endif
     }
     else
@@ -320,7 +321,7 @@ gui_buffer_search (char *server, char *channel)
     if (!ptr_buffer)
         return NULL;
     
-    return (ptr_buffer->type != BUFFER_TYPE_STANDARD) ?
+    return (ptr_buffer->type != GUI_BUFFER_TYPE_STANDARD) ?
         gui_buffers : ptr_buffer;
 }
 
@@ -462,13 +463,13 @@ gui_buffer_get_dcc (t_gui_window *window)
     /* check if dcc buffer exists */
     for (ptr_buffer = gui_buffers; ptr_buffer; ptr_buffer = ptr_buffer->next_buffer)
     {
-        if (ptr_buffer->type == BUFFER_TYPE_DCC)
+        if (ptr_buffer->type == GUI_BUFFER_TYPE_DCC)
             break;
     }
     if (ptr_buffer)
         return ptr_buffer;
     else
-        return gui_buffer_new (window, NULL, NULL, BUFFER_TYPE_DCC, 0);
+        return gui_buffer_new (window, NULL, NULL, GUI_BUFFER_TYPE_DCC, 0);
 }
 
 /*
@@ -484,7 +485,7 @@ gui_buffer_clear (t_gui_buffer *buffer)
     if (!buffer)
         return;
 
-    if (buffer->type == BUFFER_TYPE_DCC)
+    if (buffer->type == GUI_BUFFER_TYPE_DCC)
         return;
     
     /* remove buffer from hotlist */
@@ -572,12 +573,13 @@ gui_buffer_free (t_gui_buffer *buffer, int switch_to_another)
     t_irc_server *ptr_server;
     int create_new;
 #ifdef PLUGINS
-    char buffer_str[16];
+    char buffer_str[16], *argv[1] = { NULL };
 #endif
     
 #ifdef PLUGINS
     snprintf (buffer_str, sizeof (buffer_str) - 1, "%d", buffer->number);
-    (void) plugin_event_handler_exec ("buffer_close", buffer_str);
+    argv[0] = buffer_str;
+    (void) plugin_event_handler_exec ("buffer_close", 1, argv);
 #endif
     
     create_new = (buffer->server || buffer->channel);
@@ -595,6 +597,9 @@ gui_buffer_free (t_gui_buffer *buffer, int switch_to_another)
     hotlist_remove_buffer (buffer);
     if (hotlist_initial_buffer == buffer)
         hotlist_initial_buffer = NULL;
+
+    if (gui_previous_buffer == buffer)
+        gui_previous_buffer = NULL;
     
     if (gui_buffer_before_dcc == buffer)
         gui_buffer_before_dcc = NULL;
@@ -602,7 +607,7 @@ gui_buffer_free (t_gui_buffer *buffer, int switch_to_another)
     if (gui_buffer_before_raw_data == buffer)
         gui_buffer_before_raw_data = NULL;
     
-    if (buffer->type == BUFFER_TYPE_RAW_DATA)
+    if (buffer->type == GUI_BUFFER_TYPE_RAW_DATA)
         gui_buffer_raw_data = NULL;
     
     for (ptr_server = irc_servers; ptr_server;
@@ -663,7 +668,7 @@ gui_buffer_free (t_gui_buffer *buffer, int switch_to_another)
     /* always at least one buffer */
     if (!gui_buffers && create_new && switch_to_another)
         (void) gui_buffer_new (gui_windows, NULL, NULL,
-                               BUFFER_TYPE_STANDARD, 1);
+                               GUI_BUFFER_TYPE_STANDARD, 1);
     
     if (gui_windows && gui_current_window && gui_current_window->buffer)
         gui_status_draw (gui_current_window->buffer, 1);
@@ -716,7 +721,6 @@ gui_buffer_line_new (t_gui_buffer *buffer, time_t date)
         buffer->lines = ptr_line;
         ptr_line->prev_line = NULL;
         buffer->num_lines--;
-        gui_chat_draw (buffer, 1);
     }
     
     return new_line;
@@ -732,11 +736,11 @@ gui_buffer_line_search (t_gui_line *line, char *text, int case_sensitive)
     char *data;
     int rc;
     
-    if (!text || !text[0])
+    if (!line || !line->data || !text || !text[0])
         return 0;
     
     rc = 0;
-    data = (char *)gui_color_decode ((unsigned char *)line->data, 0);
+    data = (char *)gui_color_decode ((unsigned char *)line->data, 0, 0);
     if (data)
     {
         if ((case_sensitive && (strstr (data, text)))
@@ -761,7 +765,7 @@ gui_buffer_merge_servers (t_gui_window *window)
     for (ptr_buffer_server = gui_buffers; ptr_buffer_server;
          ptr_buffer_server = ptr_buffer_server->next_buffer)
     {
-        if (BUFFER_IS_SERVER(ptr_buffer_server))
+        if (GUI_BUFFER_IS_SERVER(ptr_buffer_server))
             break;
     }
     
@@ -773,9 +777,9 @@ gui_buffer_merge_servers (t_gui_window *window)
     while (ptr_buffer)
     {
         if ((ptr_buffer != ptr_buffer_server)
-            && (BUFFER_IS_SERVER(ptr_buffer)))
+            && (GUI_BUFFER_IS_SERVER(ptr_buffer)))
         {
-            ptr_server = SERVER(ptr_buffer);
+            ptr_server = GUI_SERVER(ptr_buffer);
             
             /* add (by pointer artefact) lines from buffer found to server buffer */
             if (ptr_buffer->lines)
@@ -828,18 +832,18 @@ gui_buffer_split_server (t_gui_window *window)
     
     if (ptr_buffer)
     {
-        if (SERVER(ptr_buffer))
+        if (GUI_SERVER(ptr_buffer))
         {
             for (ptr_server = irc_servers; ptr_server;
                  ptr_server = ptr_server->next_server)
             {
                 if (ptr_server->buffer
-                    && (ptr_server != SERVER(ptr_buffer))
+                    && (ptr_server != GUI_SERVER(ptr_buffer))
                     && (ptr_server->buffer == ptr_buffer))
                 {
                     ptr_server->buffer = NULL;
                     gui_buffer_new (window, ptr_server, NULL,
-                                    BUFFER_TYPE_STANDARD, 0);
+                                    GUI_BUFFER_TYPE_STANDARD, 0);
                 }
             }
         }
@@ -905,7 +909,7 @@ gui_buffer_switch_dcc (t_gui_window *window)
     /* check if dcc buffer exists */
     for (ptr_buffer = gui_buffers; ptr_buffer; ptr_buffer = ptr_buffer->next_buffer)
     {
-        if (ptr_buffer->type == BUFFER_TYPE_DCC)
+        if (ptr_buffer->type == GUI_BUFFER_TYPE_DCC)
             break;
     }
     if (ptr_buffer)
@@ -914,7 +918,7 @@ gui_buffer_switch_dcc (t_gui_window *window)
         gui_window_redraw_buffer (ptr_buffer);
     }
     else
-        gui_buffer_new (window, NULL, NULL, BUFFER_TYPE_DCC, 1);
+        gui_buffer_new (window, NULL, NULL, GUI_BUFFER_TYPE_DCC, 1);
 }
 
 /*
@@ -929,7 +933,7 @@ gui_buffer_switch_raw_data (t_gui_window *window)
     /* check if raw IRC data buffer exists */
     for (ptr_buffer = gui_buffers; ptr_buffer; ptr_buffer = ptr_buffer->next_buffer)
     {
-        if (ptr_buffer->type == BUFFER_TYPE_RAW_DATA)
+        if (ptr_buffer->type == GUI_BUFFER_TYPE_RAW_DATA)
             break;
     }
     if (ptr_buffer)
@@ -938,7 +942,7 @@ gui_buffer_switch_raw_data (t_gui_window *window)
         gui_window_redraw_buffer (ptr_buffer);
     }
     else
-        gui_buffer_new (window, NULL, NULL, BUFFER_TYPE_RAW_DATA, 1);
+        gui_buffer_new (window, NULL, NULL, GUI_BUFFER_TYPE_RAW_DATA, 1);
 }
 
 /*
@@ -982,6 +986,9 @@ gui_buffer_move_to_number (t_gui_buffer *buffer, int number)
 {
     t_gui_buffer *ptr_buffer;
     int i;
+#ifdef PLUGINS
+    char buf1_str[16], buf2_str[16], *argv[2] = { NULL, NULL };
+#endif
     
     /* if only one buffer then return */
     if (gui_buffers == last_gui_buffer)
@@ -993,6 +1000,10 @@ gui_buffer_move_to_number (t_gui_buffer *buffer, int number)
     
     if (number < 1)
         number = 1;
+
+#ifdef PLUGINS
+    snprintf (buf2_str, sizeof (buf2_str) - 1, "%d", buffer->number);
+#endif
     
     /* remove buffer from list */
     if (buffer == gui_buffers)
@@ -1060,6 +1071,13 @@ gui_buffer_move_to_number (t_gui_buffer *buffer, int number)
     }
     
     gui_window_redraw_buffer (buffer);
+    
+#ifdef PLUGINS
+    snprintf (buf1_str, sizeof (buf1_str) - 1, "%d", buffer->number);
+    argv[0] = buf1_str;
+    argv[1] = buf2_str;
+    (void) plugin_event_handler_exec ("buffer_move", 2, argv);
+#endif
 }
 
 /*
@@ -1071,7 +1089,7 @@ gui_buffer_search_text (t_gui_window *window)
 {
     t_gui_line *ptr_line;
     
-    if (window->buffer->text_search == TEXT_SEARCH_BACKWARD)
+    if (window->buffer->text_search == GUI_TEXT_SEARCH_BACKWARD)
     {
         if (window->buffer->lines
             && window->buffer->input_buffer && window->buffer->input_buffer[0])
@@ -1095,7 +1113,7 @@ gui_buffer_search_text (t_gui_window *window)
             }
         }
     }
-    else if (window->buffer->text_search == TEXT_SEARCH_FORWARD)
+    else if (window->buffer->text_search == GUI_TEXT_SEARCH_FORWARD)
     {
         if (window->buffer->lines
             && window->buffer->input_buffer && window->buffer->input_buffer[0])
@@ -1129,7 +1147,7 @@ gui_buffer_search_text (t_gui_window *window)
 void
 gui_buffer_search_start (t_gui_window *window)
 {
-    window->buffer->text_search = TEXT_SEARCH_BACKWARD;
+    window->buffer->text_search = GUI_TEXT_SEARCH_BACKWARD;
     window->buffer->text_search_exact = 0;
     window->buffer->text_search_found = 0;
     if (window->buffer->text_search_input)
@@ -1155,7 +1173,7 @@ gui_buffer_search_restart (t_gui_window *window)
 {
     window->start_line = NULL;
     window->start_line_pos = 0;
-    window->buffer->text_search = TEXT_SEARCH_BACKWARD;
+    window->buffer->text_search = GUI_TEXT_SEARCH_BACKWARD;
     window->buffer->text_search_found = 0;
     if (gui_buffer_search_text (window))
         window->buffer->text_search_found = 1;
@@ -1173,7 +1191,7 @@ gui_buffer_search_restart (t_gui_window *window)
 void
 gui_buffer_search_stop (t_gui_window *window)
 {
-    window->buffer->text_search = TEXT_SEARCH_DISABLED;
+    window->buffer->text_search = GUI_TEXT_SEARCH_DISABLED;
     window->buffer->text_search = 0;
     gui_action_delete_line (window, NULL);
     if (window->buffer->text_search_input)
@@ -1190,6 +1208,269 @@ gui_buffer_search_stop (t_gui_window *window)
     gui_input_draw (window->buffer, 1);
 }
 
+/*
+ * gui_buffer_scroll: scroll buffer by # messages or time
+ */
+
+void
+gui_buffer_scroll (t_gui_window *window, char *scroll)
+{
+    int direction, stop, count_msg;
+    char time_letter, saved_char;
+    time_t old_date, diff_date;
+    char *error;
+    long number;
+    t_gui_line *ptr_line;
+    struct tm *date_tmp, line_date, old_line_date;
+    
+    if (window->buffer->lines)
+    {
+        direction = -1;
+        number = 0;
+        time_letter = ' ';
+        
+        // search direction
+        if (scroll[0] == '-')
+        {
+            direction = -1;
+            scroll++;
+        }
+        else if (scroll[0] == '+')
+        {
+            direction = +1;
+            scroll++;
+        }
+        
+        // search number and letter
+        char *pos = scroll;
+        while (pos && pos[0] && isdigit (pos[0]))
+        {
+            pos++;
+        }
+        if (pos)
+        {
+            if (pos == scroll)
+            {
+                if (pos[0])
+                    time_letter = scroll[0];
+            }
+            else
+            {
+                if (pos[0])
+                    time_letter = pos[0];
+                saved_char = pos[0];
+                pos[0] = '\0';
+                error = NULL;
+                number = strtol (scroll, &error, 10);
+                if (!error || (error[0] != '\0'))
+                    number = 0;
+                pos[0] = saved_char;
+            }
+        }
+        
+        /* at least number or letter has to he given */
+        if ((number == 0) && (time_letter == ' '))
+            return;
+        
+        // do the scroll!
+        stop = 0;
+        count_msg = 0;
+        if (direction < 0)
+            ptr_line = (window->start_line) ?
+                window->start_line : window->buffer->last_line;
+        else
+            ptr_line = (window->start_line) ?
+                window->start_line : window->buffer->lines;
+        
+        old_date = ptr_line->date;
+        date_tmp = localtime (&old_date);
+        memcpy (&old_line_date, date_tmp, sizeof (struct tm));
+        
+        while (ptr_line)
+        {
+            ptr_line = (direction < 0) ? ptr_line->prev_line : ptr_line->next_line;
+            
+            if (ptr_line)
+            {
+                if (time_letter == ' ')
+                {
+                    count_msg++;
+                    if (count_msg >= number)
+                        stop = 1;
+                }
+                else
+                {
+                    date_tmp = localtime (&(ptr_line->date));
+                    memcpy (&line_date, date_tmp, sizeof (struct tm));
+                    if (old_date > ptr_line->date)
+                        diff_date = old_date - ptr_line->date;
+                    else
+                        diff_date = ptr_line->date - old_date;
+                    switch (time_letter)
+                    {
+                        case 's': /* seconds */
+                            if (number == 0)
+                            {
+                                /* stop if line has different second */
+                                if ((line_date.tm_sec != old_line_date.tm_sec)
+                                    || (line_date.tm_min != old_line_date.tm_min)
+                                    || (line_date.tm_hour != old_line_date.tm_hour)
+                                    || (line_date.tm_mday != old_line_date.tm_mday)
+                                    || (line_date.tm_mon != old_line_date.tm_mon)
+                                    || (line_date.tm_year != old_line_date.tm_year))
+                                    if (line_date.tm_sec != old_line_date.tm_sec)
+                                        stop = 1;
+                            }
+                            else if (diff_date >= number)
+                                stop = 1;
+                            break;
+                        case 'm': /* minutes */
+                            if (number == 0)
+                            {
+                                /* stop if line has different minute */
+                                if ((line_date.tm_min != old_line_date.tm_min)
+                                    || (line_date.tm_hour != old_line_date.tm_hour)
+                                    || (line_date.tm_mday != old_line_date.tm_mday)
+                                    || (line_date.tm_mon != old_line_date.tm_mon)
+                                    || (line_date.tm_year != old_line_date.tm_year))
+                                    stop = 1;
+                            }
+                            else if (diff_date >= number * 60)
+                                stop = 1;
+                            break;
+                        case 'h': /* hours */
+                            if (number == 0)
+                            {
+                                /* stop if line has different hour */
+                                if ((line_date.tm_hour != old_line_date.tm_hour)
+                                    || (line_date.tm_mday != old_line_date.tm_mday)
+                                    || (line_date.tm_mon != old_line_date.tm_mon)
+                                    || (line_date.tm_year != old_line_date.tm_year))
+                                    stop = 1;
+                            }
+                            else if (diff_date >= number * 60 * 60)
+                                stop = 1;
+                            break;
+                        case 'd': /* days */
+                            if (number == 0)
+                            {
+                                /* stop if line has different day */
+                                if ((line_date.tm_mday != old_line_date.tm_mday)
+                                    || (line_date.tm_mon != old_line_date.tm_mon)
+                                    || (line_date.tm_year != old_line_date.tm_year))
+                                    stop = 1;
+                            }
+                            else if (diff_date >= number * 60 * 60 * 24)
+                                stop = 1;
+                            break;
+                        case 'M': /* months */
+                            if (number == 0)
+                            {
+                                /* stop if line has different month */
+                                if ((line_date.tm_mon != old_line_date.tm_mon)
+                                    || (line_date.tm_year != old_line_date.tm_year))
+                                    stop = 1;
+                            }
+                            /* we consider month is 30 days, who will find I'm too
+                               lazy to code exact date diff ? ;) */
+                            else if (diff_date >= number * 60 * 60 * 24 * 30)
+                                stop = 1;
+                            break;
+                        case 'y': /* years */
+                            if (number == 0)
+                            {
+                                /* stop if line has different year */
+                                if (line_date.tm_year != old_line_date.tm_year)
+                                    stop = 1;
+                            }
+                            /* we consider year is 365 days, who will find I'm too
+                               lazy to code exact date diff ? ;) */
+                            else if (diff_date >= number * 60 * 60 * 24 * 365)
+                                stop = 1;
+                            break;
+                    }
+                }
+                if (stop)
+                {
+                    window->start_line = ptr_line;
+                    window->start_line_pos = 0;
+                    window->first_line_displayed =
+                        (window->start_line == window->buffer->lines);
+                    gui_chat_draw (window->buffer, 1);
+                    gui_status_draw (window->buffer, 0);
+                    return;
+                }
+            }
+        }
+        if (direction < 0)
+            gui_window_scroll_top (window);
+        else
+            gui_window_scroll_bottom (window);
+    }
+}
+
+/*
+ * gui_buffer_dump_hexa: dump content of buffer as hexa data in log file
+ */
+
+void
+gui_buffer_dump_hexa (t_gui_buffer *buffer)
+{
+    t_gui_line *ptr_line;
+    int num_line, data_pos;
+    char *data_without_colors;
+    char hexa[(16 * 3) + 1], ascii[(16 * 2) + 1];
+    int hexa_pos, ascii_pos;
+    
+    weechat_log_printf ("[buffer dump hexa (addr:0x%X)]\n", buffer);
+    num_line = 1;
+    for (ptr_line = buffer->lines; ptr_line; ptr_line = ptr_line->next_line)
+    {
+        /* display line without colors */
+        data_without_colors = (ptr_line->data) ?
+            (char *)gui_color_decode ((unsigned char *)ptr_line->data, 0, 0) : NULL;
+        weechat_log_printf ("\n");
+        weechat_log_printf ("  line %d: %s\n",
+                            num_line,
+                            (data_without_colors) ? data_without_colors : "(null)");
+        if (data_without_colors)
+            free (data_without_colors);
+
+        /* display raw data for line */
+        if (ptr_line->data)
+        {
+            weechat_log_printf ("\n");
+            weechat_log_printf ("  raw data for line %d (with color codes):\n",
+                                num_line);
+            data_pos = 0;
+            hexa_pos = 0;
+            ascii_pos = 0;
+            while (ptr_line->data[data_pos])
+            {
+                snprintf (hexa + hexa_pos, 4, "%02X ",
+                          (unsigned char)(ptr_line->data[data_pos]));
+                hexa_pos += 3;
+                snprintf (ascii + ascii_pos, 3, "%c ",
+                          ((((unsigned char)ptr_line->data[data_pos]) < 32)
+                           || (((unsigned char)ptr_line->data[data_pos]) > 127)) ?
+                          '.' : (unsigned char)(ptr_line->data[data_pos]));
+                ascii_pos += 2;
+                if (ascii_pos == 32)
+                {
+                    weechat_log_printf ("    %-48s  %s\n", hexa, ascii);
+                    hexa_pos = 0;
+                    ascii_pos = 0;
+                }
+                data_pos++;
+            }
+            if (ascii_pos > 0)
+                weechat_log_printf ("    %-48s  %s\n", hexa, ascii);
+        }
+        
+        num_line++;
+    }
+}
+
 /*
  * gui_buffer_print_log: print buffer infos in log (usually for crash dump)
  */
