@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -29,19 +28,22 @@
 
 
 #define gui_printf(buffer, fmt, argz...) \
-    gui_printf_internal(buffer, 1, MSG_TYPE_INFO, NULL, fmt, ##argz)
+    gui_printf_internal(buffer, 1, GUI_MSG_TYPE_INFO, -1, NULL, fmt, ##argz)
+
+#define gui_printf_keep_colors(buffer, fmt, argz...) \
+    gui_printf_internal(buffer, 1, GUI_MSG_TYPE_INFO, 1, NULL, fmt, ##argz)
 
 #define gui_printf_type(buffer, type, fmt, argz...) \
-    gui_printf_internal(buffer, 1, type, NULL, fmt, ##argz)
+    gui_printf_internal(buffer, 1, type, -1, NULL, fmt, ##argz)
 
 #define gui_printf_type_nick(buffer, type, nick, fmt, argz...) \
-    gui_printf_internal(buffer, 1, type, nick, fmt, ##argz)
+    gui_printf_internal(buffer, 1, type, -1, nick, fmt, ##argz)
 
 #define gui_printf_nolog(buffer, fmt, argz...) \
-    gui_printf_internal(buffer, 1, MSG_TYPE_INFO | MSG_TYPE_NOLOG, NULL, fmt, ##argz)
+    gui_printf_internal(buffer, 1, GUI_MSG_TYPE_INFO | GUI_MSG_TYPE_NOLOG, -1, NULL, fmt, ##argz)
 
 #define gui_printf_nolog_notime(buffer, fmt, argz...) \
-    gui_printf_internal(buffer, 0, MSG_TYPE_NOLOG, NULL, fmt, ##argz)
+    gui_printf_internal(buffer, 0, GUI_MSG_TYPE_NOLOG, -1, NULL, fmt, ##argz)
 
 
 typedef struct t_gui_infobar t_gui_infobar;
@@ -106,6 +108,8 @@ extern void gui_buffer_search_start (t_gui_window *);
 extern void gui_buffer_search_restart (t_gui_window *);
 extern void gui_buffer_search_stop (t_gui_window *);
 extern int gui_buffer_search_text (t_gui_window *);
+extern void gui_buffer_scroll (t_gui_window *, char *);
+extern void gui_buffer_dump_hexa (t_gui_buffer *);
 extern void gui_buffer_print_log (t_gui_buffer *);
 
 /* panel */
@@ -154,12 +158,14 @@ extern void gui_action_jump_smart (t_gui_window *, char *);
 extern void gui_action_jump_dcc (t_gui_window *, char *);
 extern void gui_action_jump_raw_data (t_gui_window *, char *);
 extern void gui_action_jump_last_buffer (t_gui_window *, char *);
+extern void gui_action_jump_previous_buffer (t_gui_window *, char *);
 extern void gui_action_jump_server (t_gui_window *, char *);
 extern void gui_action_jump_next_server (t_gui_window *, char *);
 extern void gui_action_switch_server (t_gui_window *, char *);
 extern void gui_action_scroll_previous_highlight (t_gui_window *, char *);
 extern void gui_action_scroll_next_highlight (t_gui_window *, char *);
 extern void gui_action_scroll_unread (t_gui_window *, char *);
+extern void gui_action_set_unread (t_gui_window *, char *);
 extern void gui_action_hotlist_clear (t_gui_window *, char *);
 extern void gui_action_infobar_clear (t_gui_window *, char *);
 extern void gui_action_refresh_screen (t_gui_window *, char *);
@@ -169,7 +175,8 @@ extern void gui_action_search_text (t_gui_window *, char *);
 
 /* key */
 extern void gui_keyboard_init ();
-extern void gui_keyboard_init_grab ();
+extern void gui_keyboard_grab_init ();
+extern void gui_keyboard_grab_end ();
 extern char *gui_keyboard_get_internal_code (char *);
 extern char *gui_keyboard_get_expanded_name (char *);
 extern t_gui_key *gui_keyboard_search (char *);
@@ -180,6 +187,11 @@ extern int gui_keyboard_unbind (char *);
 extern int gui_keyboard_pressed (char *);
 extern void gui_keyboard_free (t_gui_key *);
 extern void gui_keyboard_free_all ();
+extern void gui_keyboard_buffer_reset ();
+extern void gui_keyboard_buffer_add (int);
+extern int gui_keyboard_get_paste_lines ();
+extern void gui_keyboard_paste_accept ();
+extern void gui_keyboard_paste_cancel ();
 
 /* log */
 extern void gui_log_write_date (t_gui_buffer *);
@@ -191,7 +203,7 @@ extern void gui_log_end (t_gui_buffer *);
 /* other */
 extern int gui_word_strlen (t_gui_window *, char *);
 extern int gui_word_real_pos (t_gui_window *, char *, int);
-extern void gui_printf_internal (t_gui_buffer *, int, int, char *, char *, ...);
+extern void gui_printf_internal (t_gui_buffer *, int, int, int, char *, char *, ...);
 extern void gui_printf_raw_data (void *, int, int, char *);
 extern void gui_infobar_printf (int, int, char *, ...);
 extern void gui_infobar_printf_from_buffer (t_gui_buffer *, int, int, char *, char *, ...);
@@ -210,9 +222,9 @@ extern int gui_insert_string_input (t_gui_window *, char *, int);
 /* color */
 extern int gui_color_assign (int *, char *);
 extern char *gui_color_get_name (int);
-extern unsigned char *gui_color_decode (unsigned char *, int);
+extern unsigned char *gui_color_decode (unsigned char *, int, int);
 extern unsigned char *gui_color_decode_for_user_entry (unsigned char *);
-extern unsigned char *gui_color_encode (unsigned char *);
+extern unsigned char *gui_color_encode (unsigned char *, int);
 extern void gui_color_init_pairs ();
 extern void gui_color_rebuild_weechat();
 
@@ -245,6 +257,7 @@ extern int gui_window_objects_init (t_gui_window *);
 extern void gui_window_objects_free (t_gui_window *, int);
 extern int gui_window_calculate_pos_size (t_gui_window *, int);
 extern void gui_window_redraw_buffer (t_gui_buffer *);
+extern void gui_window_redraw_all_buffers ();
 extern void gui_window_switch_to_buffer (t_gui_window *, t_gui_buffer *);
 extern void gui_window_page_up (t_gui_window *);
 extern void gui_window_page_down (t_gui_window *);
