@@ -4,7 +4,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* This header is designed to be distributed with WeeChat plugins */
@@ -87,7 +86,7 @@ struct t_plugin_server_info
     int autoconnect;                /* = 1 if auto connect at startup       */
     int autoreconnect;              /* = 1 if auto reco when disconnected   */
     int autoreconnect_delay;        /* delay before trying again reconnect  */
-    int command_line;               /* server was given on command line     */
+    int temp_server;                /* server was given on command line     */
     char *address;                  /* address of server (IP or name)       */
     int port;                       /* port for server (6667 by default)    */
     int ipv6;                       /* use IPv6 protocol                    */
